/* global Posting, Tab, Hub, chrome, Config, Logger, Tracker */

/*
 * Util functions for CRUD chrome.tabs
 * Dependency
   - Config
   - shared/utils
 */
(function(root) {
    'use strict';

    var undef;
    var handlers = [];
    var onRemovedHandlers = [];
    var Utils = root.Utils;
    var closedTabs = [];
    var closedWindows = [];
    var popupTabs = [];
    var hiddenTab;
    var bLoading = false;

    function runtimeErrorHandler() {
        if(chrome.runtime.lastError) {
            Logger.error('[Runtime Error]', chrome.runtime.lastError);
        }
    }

    function getActiveTab(cb) {
        chrome.tabs.query({
            currentWindow: true,
            active: true
        }, function(tabs) {
            var tab = tabs[0];

            cb(tab);
        });
    }


    function sendMsgToActiveTab(data, cb) {
        getActiveTab(function(tab, error) {
            if(error) {
                Logger.warn('Send Message to wrong tab', data);

                if(typeof cb === "function") {
                    cb(error);
                }

                return;
            }
            sendMsgToTab(tab.id, data, cb);
        });
    }

    function sendMsgToTab(tabId, data, cb) {
        var value;

        function handler(tabId) {
            var port = chrome.tabs.connect(tabId, {
                name: 'gllue-ext-' + data.type
            });

            port.postMessage(data);

            if (cb) {
                cb(chrome.runtime.lastError || "");
            }
        }

        if(!tabId || closedTabs.indexOf(tabId) > -1) {
            try{
                value = typeof data.value === "string" ?  JSON.parse(data.value) : data.value;
            } catch(e) {
                value =  data.value;
            }

            if(value.url) {
                createTab({
                    url: value.url,
                    active: true,
                    cb: function(tab) {
                        handler(tab.id);
                    }
                });
            } else {
                Logger.error('Send Message to tab Failed!', data);
            }

            return;
        }

        handler(tabId);
    }

    function sendMsgToTabNew(tabId, data){
        var port = chrome.tabs.connect(tabId, {
            name: 'gllue-ext'
        });
        port.postMessage(data);
    }

    function sendRequestToTab(tabId, data, cb) {
        Logger.log("Send request to tab " + tabId + ", request data is:", data);

        chrome.tabs.sendRequest(tabId, data, function(response) {
            if(response.type === data.type){
                cb(response.data);
            }
        });
    }

    function onUpdate(cb) {
        handlers.push(cb);
    }

    function onRemoved(cb) {
        onRemovedHandlers.push(cb);
    }

    function showPageByFileName(filename) {
        if(bLoading) {
            return;
        }

        // open option page
        var url = chrome.extension.getURL(filename);

        bLoading = true;

        chrome.tabs.query({
            url: url
        }, function(tabs) {
            if (tabs.length) {
                chrome.tabs.update(tabs[0].id, {
                    active: true
                });
            } else {
                chrome.tabs.create({
                    url: url
                });
            }

            bLoading = false;
        });
    }


    function createTab(config) {
        return new Promise(function(resolve, reject){
            chrome.tabs.create(config, resolve);
        })
    }

    chrome.tabs.onUpdated.addListener(function(tabId, changeInfo, tab) {
        if (changeInfo.status === 'complete') {
            Utils.writeTabIdToWebPage(tab);

            handlers.forEach(function(handler) {
                handler(tabId, changeInfo, tab);
            });
        }
    });

    chrome.tabs.onRemoved.addListener(function(tabId, removeInfo) {
        if(hiddenTab && hiddenTab.id === tabId) {
            hiddenTab = null;

            // clear hidden tab update handlers
            handlers = handlers.filter(function(cb) {return cb.tabId !== tabId;});
            return;
        }

        onRemovedHandlers.forEach(function(handler){
            handler(tabId, removeInfo);
        });
        onRemovedHandlers = onRemovedHandlers.filter(function(cb) {return cb.tabId !== tabId;});

        closedTabs.push(tabId);
    });

    chrome.windows.onRemoved.addListener(function(windowId) {
        popupTabs.forEach(function(info, index) {
            if(windowId === info.windowId) {

                if(typeof popupTabs[index].cb === "function"){
                    popupTabs[index].cb(null, info.session_id);
                }

                closedWindows.push( popupTabs[index] );

                popupTabs.splice(index, 1);

                return false;
            }
        });
    });

    function removeWindowById(windowId, cb) {
        chrome.window.remove(windowId, cb);
    }

    function setTabToActive(tabId) {
        if(isNaN(tabId)) {
            return;
        }

        chrome.tabs.update(tabId, {
            active: true,
            selected: true
        }, function(tab) {
            runtimeErrorHandler();
        });
    }

    root.Tab = {
        getActiveTab: getActiveTab,
        sendMsgToTab: sendMsgToTab,
        sendMsgToActiveTab: sendMsgToActiveTab,
        sendMsgToTabnew: sendMsgToTabNew,
        onUpdate: onUpdate,
        onRemoved: onRemoved,
        showPageByFileName: showPageByFileName,
        sendRequestToTab: sendRequestToTab,
        removeWindowById: removeWindowById,
        create: createTab,
        setTabToActive: setTabToActive
    };

})(window);
