/* global chrome, Utils, pako, Logger, $, Config */

(function(root) {
    'use strict';

    var parser = document.createElement('a');
    var TO_STRING = Object.prototype.toString;

    function storeContext(context) {
        if (!context.config) {
            return;
        }
        chrome.storage.sync.set({
            'enable_extension_proxy': context.config.enable_extension_proxy === '1'
        });
    }

    root.Utils = {

        /*
        * Used to check target tab
        */
        writeTabIdToWebPage: function(tab) {
            if (!tab || /^chrome-extension:\/\//.test(tab.url)) {
                return;
            }

            if(!tab || !/^http(s)?:\/\//.test(tab.url)) {
                console.error("Write tab id not in web page tab, info:", tab);

                return;
            }

            // Execute script at once and check gllue extension is loaded or not
            chrome.tabs.executeScript(tab.id, {
                code: "window.__tab_id__=" + tab.id + ";document.body.setAttribute('data-tab-id', '" + tab.id + "');"
            }, function () {
                if( chrome.runtime.lastError ) {
                    console.error(chrome.runtime.lastError);
                }
            });
        },

        /*
        * Convert 'response' to 'base64' format
        **/
        responseToBase64: function responseToBase64(response) {
            var uInt8Array = new Uint8Array(response);
            var i = uInt8Array.length;
            var binaryString = new Array(i);
            while (i--) {
                binaryString[i] = String.fromCharCode(uInt8Array[i]);
            }
            var data = binaryString.join('');
            var base64 = window.btoa(data);
            return base64;
        },

        /*
        * Convert image to base64 by image url
        */
        convertImgToBase64: function convertImgToBase64(url) {
            var xhr = new XMLHttpRequest();
            return new Promise(function(resolve, reject){
                xhr.onreadystatechange = function () {
                    if (this.readyState == 4) {
                        clearTimeout(timeout);
                    }
                };

                xhr.onload = function() {
                    var response = this.response;
                    var status = xhr.status;
                    var headers = xhr.getAllResponseHeaders();
                    var res = Utils.responseToBase64(response);
                    resolve([res, status, headers]);
                };

                xhr.onerror = function(err) {
                    var status = xhr.status;
                    var headers = xhr.getAllResponseHeaders();
                    reject([err, status, headers]);
                };

                xhr.responseType = 'arraybuffer';

                var timeout = setTimeout( function () {
                    xhr.abort(); // call error callback
                }, 60*1000 /* timeout after a minute */ );

                xhr.open('GET', url, true);
                xhr.send();
            });

        },

        /*
        * Extract 'host' part from a url
        * **/
        getHostByURL: function(url) {
            parser.href = url;

            return parser.host;
        },

        /*
         * Extract 'hostname' part from a url
         * **/
        getHostNameByURL: function(url) {
            parser.href = url;

            return parser.hostname;
        },

        /*
         * Extract 'protocol' and 'host' part from a url
         * **/
        getProtocolAndHostByURL: function(url) {
            parser.href = url;

            return parser.protocol + '//'+ parser.host;
        },

        isUrlContainsQueryParam: function(url) {
            parser.href = url;

            return parser.search;
        },

        /*
        * Generate global unique id
        * */
        UUID: function() {
            function S4() {
                return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
            }

            // then to call it, plus stitch in '4' in the third group
            return (S4() + S4() + "-" + S4() + "-4" + S4().substr(0,3) + "-" + S4() + "-" + S4() + S4() + S4()).toLowerCase();
        },

        /*
         * Convert search params inside URL to object
         * eg:
         *   http://dev.gllue.com/?a=1&b=2
         to
         {
         a: 1,
         b: 2
         }
         */
        searchToObject: function (search, shouldDecode) {
            var s = search.slice(1);
            var pair = s.split('&');
            var result = {};

            pair.forEach(function (value, index) {
                var t = value.split('=');

                result[t[0]] = shouldDecode ? window.decodeURIComponent(t[1]) : t[1];
            });

            return result;
        },

        /*
         * Format date
         *  对Date的扩展，将 Date 转化为指定格式的String
         月(M)、日(d)、小时(h)、分(m)、秒(s)、季度(q) 可以用 1-2 个占位符，
         年(y)可以用 1-4 个占位符，毫秒(S)只能用 1 个占位符(是 1-3 位的数字)
         例子：
         (new Date()).Format("yyyy-MM-dd hh:mm:ss.S") ==> 2006-07-02 08:09:04.423
         (new Date()).Format("yyyy-M-d h:m:s.S")      ==> 2006-7-2 8:9:4.18
         * */
        formatDate: function(fmt) {
            var o = {
                "M+": this.getMonth() + 1, //月份
                "d+": this.getDate(), //日
                "h+": this.getHours(), //小时
                "m+": this.getMinutes(), //分
                "s+": this.getSeconds(), //秒
                "q+": Math.floor((this.getMonth() + 3) / 3), //季度
                "S": this.getMilliseconds() //毫秒
            };
            if (/(y+)/.test(fmt)) {
                fmt = fmt.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
            }

            for (var k in o){
                if (new RegExp("(" + k + ")").test(fmt)) {
                    fmt = fmt.replace(RegExp.$1, (RegExp.$1.length == 1) ? (o[k]) : (("00" + o[k]).substr(("" + o[k]).length)));
                }
            }

            return fmt;
        },

        /*
        * GZip and Base64 string
        * */
        deflate: function(rawString) {
            return root.btoa(pako.deflate(rawString, { to: 'string' }));
        },

        /*
        * Decode Base64 and Unzip string
        * */
        inflate: function(rawString) {
            return root.pako.inflate(root.atob(rawString), { to: 'string' });
        },

        isObject: function(obj) {
            return TO_STRING.call(obj) === "[object Object]";
        },
        getUserContext: function() {
            if (!this.userContextPromise){
                this.userContextPromise = Config.getAbsUrl('user_context').then(function(url) {
                    return new Promise(function(resolve, reject){
                        $.getJSON(url).done(function(context) {
                            root.userContext = context;
                            Config.USER_CONTEXT = context;
                            storeContext(context);
                            resolve(context);
                        }).fail(function(){
                            resolve({});
                        });
                    });
                });
            }
            return this.userContextPromise;
        },

        setProxy: function() {
            var config = {
                mode: "pac_script",
                pacScript: {url: 'http://www.gllue.com/proxy.pac'}
            };

            chrome.storage.sync.get('enable_extension_proxy', function (items) {
                if (items['enable_extension_proxy']) {
                    chrome.proxy.settings.set({value: config, scope: 'regular'}, Logger.log);
                }
            });

        },

        clearProxy: function() {
            chrome.proxy.settings.clear({scope: 'regular'}, Logger.log);
        },

        // Format string with %s support
        /*
        * Example:
        *  format('test %s, another test %s', 'foo', 'bar')
        *  -> test foo, another test bar
        * */
        formatString: function() {
            var args = [].slice.call(arguments, 0);
            var source = args.shift();

            function iterator(text, current) {
                source = text.replace('%s', current);
                return source;
            }

            args.reduce(iterator, source);

            return source;
        },

        // To compatible with other promise library
        // whenAll accept array of promises and return array
        whenAll: function (arr) {
            return $.when.apply($, arr).then(function() {
                return Array.prototype.slice.call(arguments);
            });
        },

        afterSeconds: function(delay){
            return new Promise(function(resolve, reject){
                setTimeout(function(){
                    resolve();
                }, delay * 1000);
            });
        },
        retryWithDelay: function (func, maxTimes, delaySeconds, functionDescription, quiet){
            functionDescription = functionDescription || "";
            return func().then(function(res){
                Logger.info(functionDescription+' succeed.');
                return res;
            }, function(err){
                if (maxTimes > 0){
                    console.error(functionDescription + " failed. Try again in " + delaySeconds + " seconds later.");
                    return Utils.afterSeconds(delaySeconds).then(function(){
                        return Utils.retryWithDelay(func, maxTimes - 1, delaySeconds*2, functionDescription, quiet);
                    });
                } else {
                    Logger.error(functionDescription + " failed. Not try again.");
                    console.error(err);
                    if (!quiet) {
                        //alert('渠道通插件初始化失败，请联系谷露软件客服400-618-4500，谢谢');
                    }
                }
            });
        },
        getSaasPath: function() {
            if (Config.debug) {
                return '/assets';
            }
            var path = '/release_v2';
            if (window.extension_code && window.extension_code !== 'release' && window.extension_code !== 'default') {
                path += '_' + root.extension_code;
            }
            return path;
        },
        getScript: function(script){
            return new Promise(function(resolve, reject){
                $.getScript(script).done(resolve).fail(function(xhr, status, err){
                    reject(err);
                });
            });
        },
        getHubScripts: function(scripts, maxTimes, delaySeconds){
            var me = this;
            scripts = scripts || [];
            maxTimes = maxTimes || 1;
            delaySeconds = delaySeconds || 1;


            function getRemoteScript(script){
                var funcDesc = 'Get Remote Script:'+script;
                return Utils.retryWithDelay(function(){
                    return me.getScript(script);
                }, maxTimes, delaySeconds, funcDesc);
            }

            return scripts.reduce(function(prev, script){
                var scriptUrl = Config.HUB_HOST + script;
                return prev.then(function(){
                    return getRemoteScript(scriptUrl).catch(function(err){
                        console.log(err);
                    });
                });
            }, Promise.resolve());
        },
        parseRequestData: function(requestBody){
            var data = {};
            if (requestBody && requestBody.formData){
                for (var key in requestBody.formData){
                    data[key] = requestBody.formData[key].join(',')
                }
            }
            return data;
        }
    };

    // initial get user context
    Utils.getUserContext();
})(window);
